etl_s3_root_dir = 'ml/etl-workspace'
etl_local_root_dir = 'tmpdata/etl-workspace'

etl_battery_swap_log = {
    'key': 'battery-swap-log'
}

etl_vm_exchange_count = {
    'key': 'vm-exchange-count'
}

class RawDataPathHelper():

    def __init__(self):
        self.s3_root_dir = 'ml/rawdata-workspace'
        self.local_root_dir = 'tmpdata/rawdata-workspace'

    def get_rawdata_key(self, name):
        return {
            'vm_list': 'vm-list',
            'vm_nearby': 'vm-nearby',
            'es_rate': 'es-rate'
        }[name]

    def get_es_rates_dirs(self, update_time_str=''):
        return self.get_dirs_by_name('es_rate', update_time_str=update_time_str)

    def get_vm_list_dirs(self, update_time_str=''):
        return self.get_dirs_by_name('vm_list', update_time_str=update_time_str)

    def get_dirs_by_name(self, name, update_time_str=''):
        key = self.get_rawdata_key(name)
        path = {}
        path['s3folder'] = '%s/%s/%s' % (self.s3_root_dir, key, update_time_str)
        path['local_dir'] = '%s/%s/%s' % (self.local_root_dir, key, update_time_str)
        path['s3prefix'] = '%s/%s' % (self.s3_root_dir, key)
        path['local_prefix'] = '%s/%s' % (self.local_root_dir, key)
        return path



class ETLPathHelper():

    def __init__(self):
        self.s3_root_dir = 'ml/etl-workspace'
        self.local_root_dir = 'tmpdata/etl-workspace'

    def get_etl_key(self, etl_name):
        return {
            'battery_swap_log': 'battery-swap-log',
            'vm_battery_count': 'vm-battery-count',
            'vm_exchange_count': 'vm-exchange-count',
            'vm_status': 'vm-status'
        }[etl_name]

    def get_battery_swap_log_dirs(self, dt_hour_str, update_time_str=''):
        etl_key = self.get_etl_key('battery_swap_log')
        return self.get_dirs_by_etl_key(etl_key, dt_hour_str, update_time_str=update_time_str)

    def get_vm_battery_count_dirs(self, dt_hour_str, update_time_str=''):
        etl_key = self.get_etl_key('vm_battery_count')
        return self.get_dirs_by_etl_key(etl_key, dt_hour_str, update_time_str=update_time_str)

    def get_vm_exchange_count_dirs(self, dt_hour_str, update_time_str=''):
        etl_key = self.get_etl_key('vm_exchange_count')
        return self.get_dirs_by_etl_key(etl_key, dt_hour_str, update_time_str=update_time_str)
    
    def get_vm_status_dirs(self, dt_hour_str, update_time_str=''):
        etl_key = self.get_etl_key('vm_status')
        return self.get_dirs_by_etl_key(etl_key, dt_hour_str, update_time_str=update_time_str)

    def get_dirs_by_etl_key(self, etl_key, dt_hour_str, update_time_str=''):
        path = {}
        path['s3folder'] = '%s/%s/%s/%s' % (self.s3_root_dir, etl_key, dt_hour_str, update_time_str)
        path['local_dir'] = '%s/%s/%s/%s' % (self.local_root_dir, etl_key, dt_hour_str, update_time_str)
        path['s3prefix'] = '%s/%s/%s' % (self.s3_root_dir, etl_key, dt_hour_str)
        path['local_prefix'] = '%s/%s/%s' % (self.local_root_dir, etl_key, dt_hour_str)
        return path


def get_local_battery_swap_log_dir(dt_hour_str, update_time_str):
    return '%s/%s/%s/%s' % (etl_local_root_dir, etl_battery_swap_log['key'], dt_hour_str, update_time_str)

def get_battery_swap_log_dir(dt_hour_str, update_time_str):
    return '%s/%s/%s/%s' % (etl_s3_root_dir, etl_battery_swap_log['key'], dt_hour_str, update_time_str)

def get_local_etl_vm_exchange_count_dir(dt_hour_str, update_time_str):
    return '%s/%s/%s/%s' % (etl_local_root_dir, etl_vm_exchange_count['key'], dt_hour_str, update_time_str)

def get_etl_vm_exchange_count_dir(dt_hour_str, update_time_str):
    return '%s/%s/%s/%s' % (etl_s3_root_dir, etl_vm_exchange_count['key'], dt_hour_str, update_time_str)

def get_local_battery_swap_log_prefix(dt_hour_str):
    return '%s/%s/%s' % (etl_local_root_dir, etl_battery_swap_log['key'], dt_hour_str)

def get_battery_swap_log_prefix(dt_hour_str):
    return '%s/%s/%s' % (etl_s3_root_dir, etl_battery_swap_log['key'], dt_hour_str)

def get_local_etl_vm_exchange_count_prefix(dt_hour_str):
    return '%s/%s/%s' % (etl_local_root_dir, etl_vm_exchange_count['key'], dt_hour_str)

def get_etl_vm_exchange_count_prefix(dt_hour_str):
    return '%s/%s/%s' % (etl_s3_root_dir, etl_vm_exchange_count['key'], dt_hour_str)



class JobPathHelper():

    def __init__(self):
        self.s3_root_dir = 'ml/job-workspace'
        self.local_root_dir = 'tmpdata/job-workspace'

    def get_job_key(self, job_name):
        return {
            'demand_prediction': 'demand-prediction',
            'demand_prediction_2h': 'demand-prediction-2h',
            'demand_prediction_4h': 'demand-prediction-4h',
            'demand_prediction_1d': 'demand-prediction-1d',
        }[job_name]

    def get_demand_prediction_dirs(self, dt_hour_str, update_time_str=''):
        return self.get_dirs_by_demand_prediction_key('demand_prediction', dt_hour_str, update_time_str=update_time_str)

    def get_demand_prediction_2h_dirs(self, dt_hour_str, update_time_str=''):
        return self.get_dirs_by_demand_prediction_key('demand_prediction_2h', dt_hour_str, update_time_str=update_time_str)

    def get_demand_prediction_4h_dirs(self, dt_hour_str, update_time_str=''):
        return self.get_dirs_by_demand_prediction_key('demand_prediction_4h', dt_hour_str, update_time_str=update_time_str)

    def get_demand_prediction_1d_dirs(self, dt_hour_str, update_time_str=''):
        return self.get_dirs_by_demand_prediction_key('demand_prediction_1d', dt_hour_str, update_time_str=update_time_str)

    def get_dirs_by_demand_prediction_name(self, job_name, dt_hour_str, update_time_str=''):
        job_key = self.get_job_key(job_name)
        path = {}
        path['s3folder'] = '%s/%s/%s/%s' % (self.s3_root_dir, job_key, dt_hour_str, update_time_str)
        path['local_dir'] = '%s/%s/%s/%s' % (self.local_root_dir, job_key, dt_hour_str, update_time_str)
        path['s3prefix'] = '%s/%s/%s' % (self.s3_root_dir, job_key, dt_hour_str)
        path['local_prefix'] = '%s/%s/%s' % (self.local_root_dir, job_key, dt_hour_str)
        return path
        





import boto3
import botocore
import configparser
import os
import shutil

class S3DataHelper():

    def __init__(
        self, 
        logger,
        config_file='go_servers.ini',
        host_name='ML_Learning_DEV'):
        self.logger = logger
        self.config_file = config_file
        self.host_name = host_name
        self.connect()

    def connect(self):
        self.config = configparser.ConfigParser()
        self.config.read(self.config_file)

        self.s3 = boto3.resource(
            's3',
            region_name=self.config[self.host_name]['REGION'])
        self.s3_Bucket = self.config[self.host_name]['S3Bucket']

    def list_object(self, prefix):
        response = self.s3.meta.client.list_objects_v2(
            Bucket=self.s3_Bucket,
            Prefix=prefix)
        if 'Contents' in response:
            return response['Contents']
        else:
            self.logger.warn('No contents in s3(Bucket=%s, prefix=%s)' %(self.s3_Bucket, prefix))
            return []

    def get_keys(self, prefix):
        """Get a list of all keys in an S3 bucket."""
        keys = []

        kwargs = {'Bucket': self.s3_Bucket, 'Prefix': prefix}
        while True:
            resp = self.s3.meta.client.list_objects_v2(**kwargs)
            if 'Contents' not in resp:
                break
            for obj in resp['Contents']:
                keys.append(obj['Key'])

            try:
                kwargs['ContinuationToken'] = resp['NextContinuationToken']
            except KeyError:
                break
        
        if len(keys)==0:
            self.logger.warn('No contents in s3(Bucket=%s, prefix=%s)' %(self.s3_Bucket, prefix))

        return keys

    def upload_file(self, s3path, local_file):
        self.logger.info('put %s to s3 (bucket=%s, key=%s)' % (local_file, self.s3_Bucket, s3path))

        self.s3.meta.client.upload_file(
            local_file,
            self.s3_Bucket,
            s3path)

    def upload_folder(self, s3folder, local_dir):
        '''
        upload a local folder to S3 (with success file)
        '''
        self.logger.info('put %s to s3 (bucket=%s, key=%s)' % (local_dir, self.s3_Bucket, s3folder))

        files = os.listdir(local_dir)
        err_cnt = 0
        for file in files:
            path = '%s/%s' % (local_dir, file)
            s3_path = '%s/%s' % (s3folder, file)
            try: 
                self.s3.meta.client.upload_file(
                    path,
                    self.s3_Bucket,
                    s3_path)
                self.logger.info('put %s to s3 (bucket=%s, key=%s)' % (path, self.s3_Bucket, s3_path))
            except Exception as e:
                self.logger.warning("failed to upload file %s to s3://%s/%s" % (path, self.s3_Bucket, s3_path))
                self.logger.warning(e)
                err_cnt += 1

        if err_cnt==0:
            self.s3.meta.client.put_object(
                    Bucket=self.s3_Bucket,
                    Key='%s/%s' % (s3folder, '_SUCCESS'))

    def download_file(self, s3path, local_file):
        self.logger.info('get %s from s3 (bucket=%s, key=%s)' % (local_file, self.s3_Bucket, s3path))
        try:
            self.s3.Bucket(self.s3_Bucket).download_file(s3path, local_file)
        except botocore.exceptions.ClientError as e:
            if e.response['Error']['Code'] == "404":
                self.logger.warn("The object in s3 (bucket=%s, key=%s) does not exist." % (self.s3_Bucket, s3path))
            else:
                raise

    def download_folder(self, s3folder, local_dir):
        self.logger.info('get %s from s3 (bucket=%s, key=%s)' % (local_dir, self.s3_Bucket, s3folder))

        objs = self.list_object(s3folder)
        if len(objs)==0:
            return

        self.mkdir(local_dir)
        for obj in objs:
            s3path = obj['Key']
            local_file = '%s%s'% (local_dir, s3path[len(s3folder):])
            self.download_file(s3path, local_file)

    def mkdir(self, root_dir):
        if not os.path.exists(root_dir):
            os.makedirs(root_dir)

    def rmdir(self, root_dir):
        '''
        Caution!
        It's dangerous like 'rm -rf'
        '''
        if root_dir[0]!='/':
            shutil.rmtree(root_dir)


    

    




from datetime import datetime


class ETL():

    def __init__(
        self,
        logger,
        learningdb_dynamodb_data_helper=None,
        learningdb_s3_data_helper=None):
        self.logger = logger
        self.learningdb_dynamodb_data_helper = learningdb_dynamodb_data_helper
        self.learningdb_s3_data_helper = learningdb_s3_data_helper

    def compute_vm_battery_count(self, dt_hour_str):
        # get vm status
        vm_status_log = self.learningdb_s3_data_helper.get_latest_vm_status(dt_hour_str)

        if len(vm_status_log)==0:
            self.logger.warn('No vm_status in %s' % dt_hour_str)
            return

        # aggregation by 10 minutes
        status = {}
        for log in vm_status_log:
            # map snap time by 10 minutes
            reg_snap_time = datetime.fromtimestamp(int(log['snap_time']))
            reg_snap_time = reg_snap_time.replace(minute=reg_snap_time.minute//10*10, second=0).timestamp()
            dt_str = self.learningdb_s3_data_helper.get_dt_str_by_timestamp(int(reg_snap_time))
            dt_minute_str = dt_str['dt_minute_str']
            vm_id = log['vm_id']
            if dt_minute_str in status:
                if vm_id in status[dt_minute_str]:
                    status[dt_minute_str][vm_id]['battery_single_count'] = max(
                        log['battery_single_count'],
                        status[dt_minute_str][vm_id]['battery_single_count'])
                    status[dt_minute_str][vm_id]['battery_pair_count'] = max(
                        log['battery_pair_count'],
                        status[dt_minute_str][vm_id]['battery_pair_count'])
                else:
                    status[dt_minute_str][vm_id] = ({
                        'battery_single_count': log['battery_single_count'],
                        'battery_pair_count': log['battery_pair_count']
                    })
            else:
                status[dt_minute_str] = {}
                status[dt_minute_str][vm_id] = {
                    'battery_single_count': log['battery_single_count'],
                    'battery_pair_count': log['battery_pair_count']
                }
        data = {}
        for dt_minute_str, status_log in status.items():
            hour_data = []
            for vm_id, log in status_log.items():
                hour_data.append({
                    'vm_id': vm_id,
                    'battery_single_count': log['battery_single_count'],
                    'battery_pair_count': log['battery_pair_count']
                })
            data[dt_minute_str] = hour_data
        # save to s3
        self.learningdb_s3_data_helper.write_vm_battery_count(data, dt_hour_str)

    def compute_vm_exchange_count(self, dt_hour_str):
        # get battery swap log
        battery_swap_log = self.learningdb_s3_data_helper.get_latest_battery_swap_log(dt_hour_str)
        
        if len(battery_swap_log)==0:
            self.logger.warn('No battery_swap_log in %s' % dt_hour_str)
            return

        vm = {}
        # parse as vm count
        for log in battery_swap_log:
            if 'vm_guid' not in log:
                continue
            vm_guid = log['vm_guid']
            if vm_guid in vm:
                vm[vm_guid]['swap_battery_count'] += len(log['swap_out'])
                vm[vm_guid]['swap_count'] += 1
            else:
                vm[vm_guid] = {
                    'swap_count': 1,
                    'swap_battery_count': len(log['swap_out'])
                }
        data = []
        for vm_guid in vm:
            data.append({
                'vm_guid': vm_guid,
                'swap_count': vm[vm_guid]['swap_count'],
                'swap_battery_count': vm[vm_guid]['swap_battery_count']
            })        

        # save to s3
        self.learningdb_s3_data_helper.write_vm_exchange_count(data, dt_hour_str)
        



get_vm_list = """
SELECT Id, Latitude, Longitude, LocName
FROM dbo.Vm vm
LEFT JOIN dbo.VmExt vmExt ON vm.Id = vmExt.vmId
WHERE (STATE = 1 OR STATE = 0 OR STATE = 99 OR STATE = 100)
    AND vmExt.vmId IS NULL
"""



class TableSyncer():

    def __init__(
        self,
        logger,
        goserver_data_helper=None,
        learningdb_dynamodb_data_helper=None,
        learningdb_s3_data_helper=None,
        dataplatform_data_helper=None,
        url_helper=None):
        self.logger = logger
        self.goserver_data_helper = goserver_data_helper
        self.learningdb_dynamodb_data_helper = learningdb_dynamodb_data_helper
        self.learningdb_s3_data_helper = learningdb_s3_data_helper
        self.dataplatform_data_helper = dataplatform_data_helper
        self.url_helper = url_helper

    def sync_vm_list(self):
        self.logger.info('Start sync_vm_list.')

        # 1: get vm_list from DataPlatform (RESTful API)
        vm_list = self.url_helper.query_vms()
        # 2: update vm_list to ML Database (S3) 
        self.learningdb_s3_data_helper.write_vm_list(vm_list)

        self.logger.info('Finish sync_vm_list.')

    def sync_es_rates(self):
        self.logger.info('Start sync_es_rates.')

        # 1: get es_rates from DataPlatform (RESTful API)
        es_rates = self.url_helper.query_esrates()
        # 2: update es_rates to ML Database (S3) 
        self.learningdb_s3_data_helper.write_es_rates(es_rates)

        self.logger.info('Finish sync_es_rates.')

    def sync_battery_swap_log(
        self,
        start_ts,
        end_ts = 0):
        self.logger.info('Start sync_battery_swap_log.')

        # 1: Check params
        max_ts_interval = 24 * 60 * 60 
        if end_ts==0:
            end_ts = start_ts + max_ts_interval
        if end_ts > start_ts + max_ts_interval:
            end_ts = start_ts + max_ts_interval
            self.logger.warn('end_ts may not larger than start_ts+%s' % max_ts_interval)

        # 2: Get Battery Swap Log from Data Platform (Mongo)
        battery_swap_log = self.dataplatform_data_helper.get_battery_swap_log(
            start_ts=start_ts,
            end_ts=end_ts)
        
        # 3: Update Battery Swap Log to ML Database (S3) 
        self.learningdb_s3_data_helper.write_battery_swap_log(battery_swap_log)

        self.logger.info('Finish sync_battery_swap_log.')

    def sync_vm_status(
        self,
        start_ts,
        end_ts = 0):

        self.logger.info('Start sync_vm_status.')
        
        # 1: Check params
        [start_ts, end_ts] = self.get_valid_ts(start_ts, end_ts)

        # 2: Get Vm Status from Data Platform (Mongo)
        # TODO: should use get_vm_status
        vm_status = self.dataplatform_data_helper.get_vm_status_b(
            start_ts=start_ts,
            end_ts=end_ts)

        # 3: Update Vm Status to ML Database (S3) 
        self.learningdb_s3_data_helper.write_vm_status(vm_status)

        self.logger.info('Finish sync_vm_status.')

    
    def get_valid_ts(
        self,
        start_ts,
        end_ts = 0):
        '''
        check time interval < max_ts_interval (1 day)
        '''

        max_ts_interval = 24 * 60 * 60 
        if end_ts==0:
            end_ts = start_ts + max_ts_interval
        if end_ts > start_ts + max_ts_interval:
            end_ts = start_ts + max_ts_interval
            self.logger.warn('end_ts may not larger than start_ts+%s' % max_ts_interval)

        return [start_ts, end_ts]

    



import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
import json
import configparser

import base64
import ast


def str_to_dict(str):
    str = str.replace('false', 'False')
    str = str.replace('true', 'True')
    return ast.literal_eval(str)

keycloak_json = {
    "realm": "gogoro",
    "auth-server-url": "https://auth.gogoro.com:8443",
    "resource": "machine_learning",
    "credentials": {
        "secret": "63dbf6a1-0173-4692-b733-889aa70935d3"
    }
}

class KeycloakSDK():

    def __init__(
        self,
        logger=None,
        keycloak_str=''):
        if keycloak_str=='':
            global keycloak_json
            keycloak = keycloak_json
        else:
            keycloak = json.loads(keycloak_str)
        self.realm = keycloak["realm"]
        self.auth_server_url = keycloak["auth-server-url"]
        self.resource = keycloak["resource"]
        self.secret = keycloak["credentials"]["secret"]
        self.token = None
    
    def get_access_token(self):
        # 1. service_account_login
        self.service_account_login()
        # 2. get_authorization
        self.get_authorization()
        # 3. refresh_access_token (schedule)
        #self.refresh_access_token()
        
        return self.token['access_token']

    def service_account_login(self):
        '''
        Service Account Login
        Method : basicAuth(resource, serviceConfig)
        '''

        url = '%s/auth/realms/%s/protocol/openid-connect/token' % (self.auth_server_url, self.realm )
        
        headers = {
            'Content-Type' : 'application/x-www-form-urlencoded',
            'Authorization': 'Basic %s' % base64.b64encode(bytes('%s:%s'% (self.resource, self.secret),'utf-8')).decode("utf-8")
        }
        payload = {
            'grant_type' : 'client_credentials'
        }
        # print('url=%s' % url)
        # print('header=%s' % headers)
        # print('data=%s' % payload)
        response = requests.post(url, headers=headers, data=payload, verify=False).text
        
        self.token = str_to_dict(response)

    def get_authorization(self): 
        '''
        GetAuthorization
        Description: Use following information to get authorization. resource 為目標要取得的 application resource
        '''
        
        url = '%s/auth/realms/%s/protocol/openid-connect/token' % (self.auth_server_url, self.realm) 
        
        headers = {
            'Content-Type' : 'application/x-www-form-urlencoded',
            'Authorization': 'Bearer %s' % self.token['access_token']
        }
        payload = {
            'grant_type' : 'urn:ietf:params:oauth:grant-type:uma-ticket',
            'audience' : self.resource
        }
        # print('url=%s' % url)
        # print('header=%s' % headers)
        # print('data=%s' % payload)
        response = requests.post(url, headers=headers, data=payload, verify=False).text
        
        self.token = str_to_dict(response)

    def refresh_access_token(self):
        '''
        RefreshGrant
        Method refreshGrant(grant)
        '''

        url = '%s/auth/realms/%s/protocol/openid-connect/token' % (self.auth_server_url, self.realm) 
        
        headers = {
            'Content-Type' : 'application/x-www-form-urlencoded',
            'Authorization': 'Basic %s' % base64.b64encode(bytes('%s:%s'% (self.resource, self.secret),'utf-8')).decode("utf-8")
        }
        payload = {
            'client_id': self.resource,
            'refresh_token' : self.token['refresh_token'],
            'grant_type' : 'refresh_token'
        }
        # print('url=%s' % url)
        # print('header=%s' % headers)
        # print('data=%s' % payload)
        response = requests.post(url, headers=headers, data=payload, verify=False).text
        
        self.token = str_to_dict(response)


class URLHelper():
    def __init__(
        self,
        logger=None,
        config_file='go_servers.ini',
        host_name='DataPlatform_DEV_API'):
        self.logger = logger
        self.config_file = config_file
        self.host_name = host_name

        self.config = configparser.ConfigParser()
        self.config.read(self.config_file)

        self.keycloak = KeycloakSDK(
            logger=logger, 
            keycloak_str=self.config[self.host_name]['keycloak'])


    def get_access_token(self):
        return self.keycloak.get_access_token()

    def query_vms(self):
        return self.query_vms_v1()

    def query_vms_nearby(self):
        return self.query_vms_nearby_v1()

    def query_vms_nearby_v1(self):
        url = '%s/v1/vms/nearby-vms' % self.config[self.host_name]['server_url']
         
        self.access_token = self.get_access_token()

        headers = {
            'Go-Client':self.keycloak.resource,
            'Authorization':'Bearer {}'.format(self.access_token),
        }
        json_obj = {
            'op_code': 'get',
            'get_data': {
                'fields_type': 'detail'
            }
        }
        json_option = None
        # print('url=%s' % url)
        # print('header=%s' % headers)
        # print('json=%s' % json_obj)
        r = requests.post(url, headers=headers, json=json_obj)
        print(r.content)
        if json_option is None:
            json_option = {}
        return json.loads(r.content.decode('utf-8'),**json_option)

    def query_vms_v1(self):
        url = '%s/v1/vms' % self.config[self.host_name]['server_url']

        self.access_token = self.get_access_token()

        headers = {
            'Go-Client':self.keycloak.resource,
            'Authorization':'Bearer {}'.format(self.access_token),
        }
        json_obj = {
            'op_code': 'get',
            'get_data': {
                'fields_type': 'detail'
            }
        }
        json_option = None
        # print('url=%s' % url)
        # print('header=%s' % headers)
        # print('json=%s' % json_obj)
        r = requests.post(url, headers=headers, json=json_obj)
        # print(r)
        if json_option is None:
            json_option = {}
        res = json.loads(r.content.decode('utf-8'),**json_option)

        if res['code']==0:
            return res['data']
        else:
            self.logger.warn('Failed to query_vms_v1: %s', res)

    def query_esrates(self):
        return self.query_esrates_v1()

    def query_esrates_v1(self):
        url = '%s/v1/es-rates' % self.config[self.host_name]['server_url']

        self.access_token = self.get_access_token()

        headers = {
            'Go-Client':self.keycloak.resource,
            'Authorization':'Bearer {}'.format(self.access_token),
        }
        json_obj = {
            'op_code': 'get',
            'get_data': {
                'get_flag': 1
            }
        }
        json_option = None
        # print('url=%s' % url)
        # print('header=%s' % headers)
        # print('json=%s' % json_obj)
        r = requests.post(url, headers=headers, json=json_obj)
        # print(r)
        if json_option is None:
            json_option = {}
        res = json.loads(r.content.decode('utf-8'),**json_option)

        if res['code']==0:
            return res['data']
        else:
            self.logger.warn('Failed to query_vms_v1: %s', res)





import pymongo
from pymongo import MongoClient
import configparser
import json
from uuid import UUID
from bson import ObjectId

class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, UUID):
            # if the obj is uuid, we simply return the value of uuid
            #return obj.hex
            return str(obj)
        elif isinstance(obj, ObjectId):
            return str(obj)
        return json.JSONEncoder.default(self, obj)

class MongoDBDataHelper():

    def __init__(
        self, 
        logger=None,
        config_file='go_servers.ini',
        host_name='DataPlatform_DEV_ReadOnly'):
        self.logger = logger
        self.config_file = config_file
        self.host_name = host_name
        self.connect()

    def connect(self):
        self.config = configparser.ConfigParser()
        self.config.read(self.config_file)

        if 'AuthMech' in self.config[self.host_name]:
            self.client = MongoClient(
                host=self.config[self.host_name]['Server'], 
                username=self.config[self.host_name]['Username'],
                password=self.config[self.host_name]['Password'],
                port=int(self.config[self.host_name]['PORT']),
                authSource=self.config[self.host_name]['Database'],
                authMechanism=self.config[self.host_name]['AuthMech'])
        else:
            self.client = MongoClient(
                host=self.config[self.host_name]['Server'], 
                port=int(self.config[self.host_name]['PORT']))
        if self.logger != None:
            self.logger.info('client = %s' % self.client)

        if 'Database' in self.config[self.host_name]:
            self.db = self.client[self.config[self.host_name]['Database']]
        else:
            self.db = None

        return self.client, self.db

    def get_collection(self, collection):
        return self.db[collection]

    def find(self, collection, condition):
        self.logger_info('start finding result for ... ')
        self.logger_info('\tdb = %s' % self.db)
        self.logger_info('\tcollection = %s' % collection)
        self.logger_info('\tcondition = %s' % condition)

        cursor = self.db[collection].find(condition)

        result = []
        for c in cursor:
            # make UUID and ObjectID JSON serializable
            result.append(json.loads(json.dumps(c, cls=JSONEncoder)))

        self.logger_info('result.count = %s' % len(result))
        if len(result) > 0:
            self.logger_info('result[0] = %s' % result[0])

        return result

    def logger_info(self, msg):
        if self.logger != None:
            self.logger.info(msg)



from datahelper.mongodb.mongodb_data_helper import MongoDBDataHelper
from datahelper.mongodb.data_platform_dev import find as Finder

class DataHelper(MongoDBDataHelper):

    def __init__(
        self, 
        logger=None,
        config_file='go_servers.ini',
        host_name='DataPlatform_DEV_ReadOnly'):
        super().__init__( 
            logger=logger,
            config_file=config_file,
            host_name=host_name)

    def connect(self):
        super().connect()

    def get_battery_swap_log_b(self, start_ts, end_ts=0):
        return super().find(
            Finder.CollectionNames().battery_swap_log_b, 
            Finder.get_battery_swap_log(start_ts, end_ts))

    def get_battery_swap_log(self, start_ts, end_ts=0):
        return super().find(
            Finder.CollectionNames().battery_swap_log, 
            Finder.get_battery_swap_log(start_ts, end_ts))

    def get_vm_status_b(self, start_ts, end_ts=0):
        return super().find(
            Finder.CollectionNames().vm_status_b, 
            Finder.get_vm_status(start_ts, end_ts))

    def get_vm_status(self, start_ts, end_ts=0):
        return super().find(
            Finder.CollectionNames().vm_status, 
            Finder.get_vm_status(start_ts, end_ts))



from datahelper.s3.s3_data_helper import S3DataHelper
from datahelper.s3.ml_learning_dev import s3_path_helper as S3PathHelper
from datahelper.s3.ml_learning_dev.s3_path_helper import ETLPathHelper, JobPathHelper, RawDataPathHelper
from datetime import datetime
import json
import os

class DataHelper(S3DataHelper):

    def __init__(
        self, 
        logger=None,
        config_file='go_servers.ini',
        host_name='ML_Learning_DEV'):
        super().__init__( 
            logger=logger,
            config_file=config_file,
            host_name=host_name)

    def connect(self):
        super().connect()

    #####################################################
    #
    #   updated data
    #
    #####################################################

    def get_latest_s3key(self, keys):
        if len(keys)==0:
            return None

        keys.sort(key=lambda x:x, reverse=True)
        for i in range(0, len(keys)):
            # if (Not _SUCCESS file) and (have corresponding _SUCCESS file)
            s3key = keys[i]
            s3_success_key = '%s%s' % ('/'.join(s3key.split('/')[:-1]), '/_SUCCESS')
            if s3key[-8:]!='_SUCCESS' and s3_success_key in keys:
                return s3key

        return None

    def write_es_rates(self, data_dict):
        if len(data_dict)==0:
            self.logger.warn('Empty input of es rate')
            return

        # upload data to s3
        dt_now_str = self.get_dt_str_by_timestamp(int(datetime.now().timestamp()))['dt']
        path = RawDataPathHelper().get_es_rates_dirs(update_time_str=dt_now_str)

        # write local file and upload
        self.mkdir(path['local_dir'])

        json_data = {}
        json_data['data'] = data_dict
        with open('%s/%s.json' % (path['local_dir'], dt_now_str), 'w') as fp:
            json.dump(json_data, fp)
        # upload
        self.upload_folder(path['s3folder'], path['local_dir'])
        # remove local file
        self.rmdir(path['local_dir']) 


    def write_vm_list(self, data_dict):
        if len(data_dict)==0:
            self.logger.warn('Empty input of vm list')
            return

        # upload data to s3
        dt_now_str = self.get_dt_str_by_timestamp(int(datetime.now().timestamp()))['dt']
        path = RawDataPathHelper().get_vm_list_dirs(update_time_str=dt_now_str)

        # write local file and upload
        self.mkdir(path['local_dir'])

        json_data = {}
        json_data['data'] = data_dict
        with open('%s/%s.json' % (path['local_dir'], dt_now_str), 'w') as fp:
            json.dump(json_data, fp)
        # upload
        self.upload_folder(path['s3folder'], path['local_dir'])
        # remove local file
        self.rmdir(path['local_dir'])   

    def get_es_rates_s3folder(self):
        # get key
        path = RawDataPathHelper().get_es_rates_dirs()
        prefix = path['s3prefix']
        keys = self.get_keys(prefix)
        s3path = self.get_latest_s3key(keys)

        if s3path == None:
            self.logger.warn('No es_rates')
            return []
        else:
            s3folder = '/'.join(s3path.split('/')[:-1])
            self.logger.info('get es_rates s3folder = %s' % s3folder)
            return s3folder

    def get_vm_list_s3folder(self):
        # get key
        path = RawDataPathHelper().get_vm_list_dirs()
        prefix = path['s3prefix']
        keys = self.get_keys(prefix)
        s3path = self.get_latest_s3key(keys)

        if s3path == None:
            self.logger.warn('No vm_list')
            return []
        else:
            s3folder = '/'.join(s3path.split('/')[:-1])
            self.logger.info('get vm_list s3folder = %s' % s3folder)
            return s3folder

    def get_es_rates(self):
        s3folder = self.get_es_rates_s3folder()

        if s3folder is None:
            self.logger.warn('No es_rates')
            return []
        
        path = RawDataPathHelper().get_es_rates_dirs(
            update_time_str=s3folder.split('/')[-1])

        self.download_folder(path['s3folder'], path['local_dir'])

        # read local files
        files = os.listdir(path['local_dir'])
        data = None
        for file in files:
            if file[-5:] != '.json':
                continue
            local_path = '%s/%s' % (path['local_dir'], file)
            with open(local_path) as fp:
                json_data = json.load(fp)
                data = json_data['data']

        # remove local files
        self.rmdir(path['local_dir'])

        self.logger.info('Read es_rates = %s' % data)
        return data

    def get_vm_list(self):
        s3folder = self.get_vm_list_s3folder()

        if s3folder is None:
            self.logger.warn('No vm_list')
            return []
        
        path = RawDataPathHelper().get_vm_list_dirs(
            update_time_str=s3folder.split('/')[-1])

        self.download_folder(path['s3folder'], path['local_dir'])

        # read local files
        files = os.listdir(path['local_dir'])
        data = None
        for file in files:
            if file[-5:] != '.json':
                continue
            local_path = '%s/%s' % (path['local_dir'], file)
            with open(local_path) as fp:
                json_data = json.load(fp)
                data = json_data['data']

        # remove local files
        self.rmdir(path['local_dir'])

        self.logger.info('Read %s logs from vm_list' % (len(data)))
        self.logger.info('vm_list[0] = %s' % data[0])
        return data

    #####################################################
    #
    #   jobs
    #
    #####################################################

    def filter_by_time(self, keys, filter_time_str):
        filter_keys = []
        for key in keys:
            time_str = key.split('/')[-2]
            if datetime.strptime(time_str, '%Y-%m-%dT%H:%M:%S') <= datetime.strptime(filter_time_str, '%Y-%m-%dT%H:%M:%S'):
                filter_keys.append(key)
        return filter_keys

    def get_latest_job_s3key(self, job_keys):
        if len(job_keys)==0:
            return None
        job_keys.sort(key=lambda x:x, reverse=True)
        for i in range(0, len(job_keys)):
            # if (Not _SUCCESS file) and (have corresponding _SUCCESS file)
            s3key = job_keys[i]
            s3_success_key = '%s%s' % ('/'.join(s3key.split('/')[:-1]), '/_SUCCESS')
            if s3key[-8:]!='_SUCCESS' and s3_success_key in job_keys:
                return s3key

        return None 

    def get_demand_prediction_result_s3folder(self, job_name, dt_hour_str, update_time_str=''):
        '''
        get latest (before 'update_time_str') demand prediction result in given 'dt_hour_str'
        '''
        job_path = JobPathHelper().get_dirs_by_demand_prediction_name(job_name, dt_hour_str)
        job_prefix = job_path['s3prefix']
        job_keys = self.get_keys(job_prefix)
        if len(job_keys)==0:
            return None
        if update_time_str!='':
            job_keys = self.filter_by_time(job_keys, update_time_str)
        s3path = self.get_latest_job_s3key(job_keys)
        # return a folder 
        s3folder = s3path[:s3path.rfind('/')]
        update_time_str = s3folder.split('/')[-1]
        self.logger.info('get demand_prediction_result s3folder = %s' % s3folder)
        return s3folder

    def download_demand_prediction_result_by_job_name(
        self, 
        job_name, 
        dt_hour_str, 
        update_time_str=''):

        s3folder = self.get_demand_prediction_result_s3folder(
            job_name,
            dt_hour_str,
            update_time_str=update_time_str)

        if s3folder is None:
            if update_time_str=='':
                self.logger.warn('No demand_prediction_result in %s for %s' % (dt_hour_str, job_name))
            else:
                self.logger.warn('No demand_prediction_result in %s for %s (before %s)' % (dt_hour_str, job_name, update_time_str))
            return []

        job_path = JobPathHelper().get_dirs_by_demand_prediction_name(
            job_name,
            dt_hour_str, 
            update_time_str=s3folder.split('/')[-1])
        
        self.download_folder(job_path['s3folder'], job_path['local_dir'])

        return job_path['local_dir']

    def upload_demand_prediction_result_by_job_name(
        self, 
        job_name,
        dt_hour_str, 
        local_dir):

        update_time_str = self.get_dt_str_by_timestamp(int(datetime.now().timestamp()))['dt']
        job_path = JobPathHelper().get_dirs_by_demand_prediction_name(
            job_name,
            dt_hour_str, 
            update_time_str=update_time_str)

        # upload
        self.upload_folder(job_path['s3folder'], local_dir)

    def download_demand_prediction_result(self, dt_hour_str, update_time_str=''):
        s3folder = self.get_demand_prediction_result_s3folder(
            dt_hour_str,
            update_time_str=update_time_str)

        if s3folder is None:
            if update_time_str=='':
                self.logger.warn('No demand_prediction_result in %s' % dt_hour_str)
            else:
                self.logger.warn('No demand_prediction_result in %s (before %s)' % (dt_hour_str, update_time_str))
            return []

        job_path = JobPathHelper().get_demand_prediction_dirs(
            dt_hour_str, 
            update_time_str=s3folder.split('/')[-1])
        
        self.download_folder(job_path['s3folder'], job_path['local_dir'])

        return job_path['local_dir']

    def upload_demand_prediction_result(self, dt_hour_str, local_dir):
        update_time_str = self.get_dt_str_by_timestamp(int(datetime.now().timestamp()))['dt']
        job_path = JobPathHelper().get_demand_prediction_dirs(
            dt_hour_str, 
            update_time_str=update_time_str)

        # upload
        self.upload_folder(job_path['s3folder'], local_dir)

    #####################################################
    #
    #   etl
    #
    #####################################################

    def get_latest_etl_s3key(self, etl_keys):
        if len(etl_keys)==0:
            return None

        etl_keys.sort(key=lambda x:x, reverse=True)
        for i in range(0, len(etl_keys)):
            # if (Not _SUCCESS file) and (have corresponding _SUCCESS file)
            s3key = etl_keys[i]
            s3_success_key = '%s%s' % ('/'.join(s3key.split('/')[:-1]), '/_SUCCESS')
            if s3key[-8:]!='_SUCCESS' and s3_success_key in etl_keys:
                return s3key

        return None

    #####################################################
    # Vm Status
    #####################################################

    def get_latest_vm_status_s3folder(self, dt_hour_str):
        job_path = ETLPathHelper().get_vm_status_dirs(dt_hour_str)
        job_prefix = job_path['s3prefix']
        etl_keys = self.get_keys(job_prefix)
        s3path = self.get_latest_etl_s3key(etl_keys)
        # return a folder 
        if s3path == None:
            self.logger.warn('No latest battery_swap_log')
            return None
        else:
            s3folder = '/'.join(s3path.split('/')[:-1])
            self.logger.info('get latest vm_status s3folder = %s' % s3folder)
            return s3folder

    def get_latest_vm_status(self, dt_hour_str):
        s3folder = self.get_latest_vm_status_s3folder(dt_hour_str)
        if s3folder is None:
            self.logger.warn('No latest_vm_status in %s', dt_hour_str)
            return []

        job_path = ETLPathHelper().get_vm_status_dirs(
            dt_hour_str, 
            update_time_str=s3folder.split('/')[-1])

        self.download_folder(job_path['s3folder'], job_path['local_dir'])

        # read local files
        files = os.listdir(job_path['local_dir'])
        data = []
        for file in files:
            if file[-5:] != '.json':
                continue
            path = '%s/%s' % (job_path['local_dir'], file)
            with open(path) as fp:
                json_data = json.load(fp)
                #data.extend(json_data['data'])
                data = json_data['data']

        # remove local files
        self.rmdir(job_path['local_dir'])

        self.logger.info('Read %s logs from vm_status in %s' % (len(data), dt_hour_str))
        self.logger.info('vm_status[0] = %s' % data[0])
        return data

    def write_vm_status(self, item_dict_list):
        if (len(item_dict_list))==0:
            self.logger.warn('Empty input of vm status')
            return
        # data preprocessing
        dict_dt_hour = {}
        for item_dict in item_dict_list:
            dt_hour_str = self.get_dt_str_by_timestamp(item_dict['snap_time'])['dt_hour_str']
            #[_, dt_hour_str, _] = self.timestamp_to_dt_str(item_dict['snap_time'])
            if dt_hour_str in dict_dt_hour:
                dict_dt_hour[dt_hour_str].append(item_dict)
            else:
                dict_dt_hour[dt_hour_str] = [item_dict]

        # upload data to s3
        dt_now_str = self.get_dt_str_by_timestamp(int(datetime.now().timestamp()))['dt']
        #[dt_now_str, _, _] = self.timestamp_to_dt_str(int(datetime.now().timestamp()))
        for dt_hour_str, item_list in dict_dt_hour.items():
            etl_key = ETLPathHelper().get_etl_key('vm_status')

            # get latest data and join with current item list
            new_list = item_list + self.get_latest_vm_status(dt_hour_str)
            # remove duplicate
            new_list = list({i['_id']:i for i in new_list}.values())
            self.logger.info('write %s vm_status in %s' % (len(new_list), dt_hour_str))
            
            self.write_etl_data(new_list, dt_hour_str, etl_key)
            # etl_path = ETLPathHelper().get_vm_status_dirs(dt_hour_str, update_time_str=dt_now_str)

            # # write local file and upload
            # self.mkdir(etl_path['local_dir'])
            # json_data = {}
            # json_data['data'] = item_list
            # with open('%s/%s.json' % (etl_path['local_dir'], dt_now_str), 'w') as fp:
            #     json.dump(json_data, fp)
            # # upload
            # self.upload_folder(etl_path['s3folder'], etl_path['local_dir'])
            # # remove local file
            # self.rmdir(etl_path['local_dir'])   

    #####################################################
    # general sync table (by Vm-Hour)
    #####################################################

    def get_latest_battery_swap_log_s3folder(self, dt_hour_str):
        job_path = ETLPathHelper().get_battery_swap_log_dirs(dt_hour_str)
        job_prefix = job_path['s3prefix']
        etl_keys = self.get_keys(job_prefix)
        s3path = self.get_latest_etl_s3key(etl_keys)
        # return a folder 
        if s3path == None:
            self.logger.warn('No latest battery_swap_log')
            return None
        else:
            s3folder = '/'.join(s3path.split('/')[:-1])
            self.logger.info('get latest battery_swap_log s3folder = %s' % s3folder)
            return s3folder

    def get_latest_battery_swap_log(self, dt_hour_str):
        s3folder = self.get_latest_battery_swap_log_s3folder(dt_hour_str)
        if s3folder is None:
            self.logger.warn('No latest_battery_swap_log in %s', dt_hour_str)
            return []

        job_path = ETLPathHelper().get_battery_swap_log_dirs(
            dt_hour_str, 
            update_time_str=s3folder.split('/')[-1])

        self.download_folder(job_path['s3folder'], job_path['local_dir'])

        # read local files
        files = os.listdir(job_path['local_dir'])
        data = []
        for file in files:
            if file[-5:] != '.json':
                continue
            path = '%s/%s' % (job_path['local_dir'], file)
            with open(path) as fp:
                json_data = json.load(fp)
                data = json_data['data']

        # remove local files
        self.rmdir(job_path['local_dir'])

        self.logger.info('Read %s logs from battery_swap_log in %s' % (len(data), dt_hour_str))
        self.logger.info('battery_swap_log[0] = %s' % data[0])
        return data

    def write_battery_swap_log(self, item_dict_list):
        if (len(item_dict_list))==0:
            self.logger.warn('Empty input of battey swap log')
            return
        # data preprocessing
        dict_dt_hour = {}
        for item_dict in item_dict_list:
            dt_hour_str = self.get_dt_str_by_timestamp(item_dict['exchange_time'])['dt_hour_str']
            #[_, dt_hour_str, _] = self.timestamp_to_dt_str(item_dict['exchange_time'])
            if dt_hour_str in dict_dt_hour:
                dict_dt_hour[dt_hour_str].append(item_dict)
            else:
                dict_dt_hour[dt_hour_str] = [item_dict]
        
        # upload data to s3
        for dt_hour_str, item_list in dict_dt_hour.items():
            etl_key = ETLPathHelper().get_etl_key('battery_swap_log')
            # get latest data and join with current item list
            new_list = item_list + self.get_latest_battery_swap_log(dt_hour_str)
            # remove duplicate
            new_list = list({i['_id']:i for i in new_list}.values())    # TODO: should adopt 'swap_id' later
            self.logger.info('write %s battery_swap_log in %s' % (len(new_list), dt_hour_str))
            self.write_etl_data(new_list, dt_hour_str, etl_key)

    #####################################################
    # general etl (by Vm-Hour)
    #####################################################

    def get_latest_vm_battery_count_s3folder(self, dt_hour_str):
        job_path = ETLPathHelper().get_vm_battery_count_dirs(dt_hour_str)
        job_prefix = job_path['s3prefix']
        etl_keys = self.get_keys(job_prefix)
        s3path = self.get_latest_etl_s3key(etl_keys)
        # return a folder 
        if s3path == None:
            self.logger.warn('No latest battery_swap_log')
            return None
        else:
            s3folder = '/'.join(s3path.split('/')[:-1])
            self.logger.info('get latest vm_battery_count s3folder = %s' % s3folder)
            return s3folder

    def get_latest_vm_exchange_count_s3folder(self, dt_hour_str):
        job_path = ETLPathHelper().get_vm_exchange_count_dirs(dt_hour_str)
        job_prefix = job_path['s3prefix']
        etl_keys = self.get_keys(job_prefix)
        s3path = self.get_latest_etl_s3key(etl_keys)
        # return a folder 
        if s3path == None:
            self.logger.warn('No latest battery_swap_log')
            return None
        else:
            s3folder = '/'.join(s3path.split('/')[:-1])
            self.logger.info('get latest vm_exchange_count s3folder = %s' % s3folder)
            return s3folder
    
    def get_latest_vm_battery_count(self, dt_hour_str):
        s3folder = self.get_latest_vm_battery_count_s3folder(dt_hour_str)

        if s3folder is None:
            self.logger.warn('No latest_vm_battery_count in %s', dt_hour_str)
            return []

        job_path = ETLPathHelper().get_vm_battery_count_dirs(
            dt_hour_str, 
            update_time_str=s3folder.split('/')[-1])

        self.download_folder(job_path['s3folder'], job_path['local_dir'])
        
        # read local files
        files = os.listdir(job_path['local_dir'])
        data = None
        for file in files:
            if file[-5:] != '.json':
                continue
            path = '%s/%s' % (job_path['local_dir'], file)
            with open(path) as fp:
                json_data = json.load(fp)
                data = json_data['data']

        # remove local files
        self.rmdir(job_path['local_dir'])

        self.logger.info('Read %s logs from vm_battery_count in %s' % (len(data), dt_hour_str))
        self.logger.info('vm_battery_count.keys() = %s' % list(data.keys()))
        return data 

    def get_latest_vm_exchange_count(self, dt_hour_str):
        s3folder = self.get_latest_vm_exchange_count_s3folder(dt_hour_str)

        if s3folder is None:
            self.logger.warn('No latest_vm_exchange_count in %s', dt_hour_str)
            return []

        job_path = ETLPathHelper().get_vm_exchange_count_dirs(
            dt_hour_str, 
            update_time_str=s3folder.split('/')[-1])

        self.download_folder(job_path['s3folder'], job_path['local_dir'])

        # read local files
        files = os.listdir(job_path['local_dir'])
        data = []
        for file in files:
            if file[-5:] != '.json':
                continue
            path = '%s/%s' % (job_path['local_dir'], file)
            with open(path) as fp:
                json_data = json.load(fp)
                data = json_data['data']

        # remove local files
        self.rmdir(job_path['local_dir'])

        self.logger.info('Read %s logs from vm_exchange_count in %s' % (len(data), dt_hour_str))
        self.logger.info('vm_exchange_count[0] = %s' % data[0])
        return data 

    def write_vm_battery_count(self, data, dt_hour_str):
        etl_key = ETLPathHelper().get_etl_key('vm_battery_count')
        self.write_etl_data(data, dt_hour_str, etl_key)

    def write_vm_exchange_count(self, data, dt_hour_str):
        etl_key = ETLPathHelper().get_etl_key('vm_exchange_count')
        self.write_etl_data(data, dt_hour_str, etl_key)

    def write_etl_data(self, data, dt_hour_str, etl_key):
        dt_now_str = self.get_dt_str_by_timestamp(int(datetime.now().timestamp()))['dt']
        etl_path = ETLPathHelper().get_dirs_by_etl_key(etl_key, dt_hour_str, update_time_str=dt_now_str)
        # write local file and upload
        self.mkdir(etl_path['local_dir'])
        json_data = {}
        json_data['data'] = data
        with open('%s/%s.json' % (etl_path['local_dir'], dt_now_str), 'w') as fp:
            json.dump(json_data, fp)
        # upload
        self.upload_folder(etl_path['s3folder'], etl_path['local_dir'])
        # remove local file
        self.rmdir(etl_path['local_dir'])

    
    #####################################################
    #
    #   general
    #
    #####################################################

    def timestamp_to_dt_str(self, timestamp):
        '''
        Return a date_string and time_string by given timestamp
        '''
        dt_str = datetime.utcfromtimestamp(timestamp).strftime('%Y-%m-%dT%H:%M:%S')
        dt_hour_str = dt_str[:13]
        time_str = dt_str[-8:]
        return [dt_str, dt_hour_str, time_str]

    def get_dt_str_by_timestamp(self, timestamp):
        '''
        Return a string format dict by given timestamp
        '''
        dt_str = {}
        dt_str['dt'] = datetime.utcfromtimestamp(timestamp).strftime('%Y-%m-%dT%H:%M:%S')
        dt_str['dt_hour_str'] = dt_str['dt'][:13]
        dt_str['dt_minute_str'] = dt_str['dt'][:16]
        dt_str['time_str'] = dt_str['dt'][-8:]
        return dt_str

    



import boto3
import configparser

class DynamoDBDataHelper():

    def __init__(
        self, 
        logger=None,
        config_file='go_servers.ini',
        host_name='ML_Learning_DEV'):
        self.logger = logger
        self.config_file = config_file
        self.host_name = host_name
        self.connect()

    def connect(self):
        self.config = configparser.ConfigParser()
        self.config.read(self.config_file)
        self.dynamodb = boto3.client(
            'dynamodb',
            region_name=self.config[self.host_name]['REGION'])

    def get_table(self, table_name):
        return boto3.resource(
                'dynamodb',
                region_name=self.config[self.host_name]['REGION']).Table(table_name)

    def create_table(self, table_dict):
        table = self.dynamodb.create_table(
            AttributeDefinitions=table_dict['AttributeDefinitions'],
            TableName=table_dict['TableName'],
            KeySchema=table_dict['KeySchema'],
            ProvisionedThroughput=table_dict['ProvisionedThroughput']
        )

        if self.logger != None:
            self.logger.info('Create Table: %s' % table)
        
        return table

    def update_table(self, table_dict):
        table = self.dynamodb.update_table(
            AttributeDefinitions=table_dict['AttributeDefinitions'],
            TableName=table_dict['TableName'],
            #KeySchema=table_dict['KeySchema'],
            ProvisionedThroughput=table_dict['ProvisionedThroughput']
        )

        if self.logger != None:
            self.logger.info('Update Table: %s' % table)
        
        return table

    def delete_table(self, table_name):
        table = self.dynamodb.delete_table(
            TableName=table_name
        )

        if self.logger != None:
            self.logger.info('Delete Table: %s' % table)

        return table
    
    def get_table_metadata(table_name):
        """
         Get some metadata about chosen table.
        """
        table = self.get_table(table_name)

        return {
            'num_items': table.item_count,
            'primary_key_name': table.key_schema[0],
            'status': table.table_status,
            'bytes_size': table.table_size_bytes,
            'global_secondary_indices': table.global_secondary_indexes
        }

    def query_table(self, table_name, filter_key=None, filter_value=None):
        """
        Perform a query operation on the table. 
        Can specify filter_key (col name) and its value to be filtered.
        """
        table = self.get_table(table_name)

        if self.logger != None:
            self.logger.info('Query Table %s with Key=%s and value=%s: %s' % (table, filter_key, filter_value))


        if filter_key and filter_value:
            filtering_exp = Key(filter_key).eq(filter_value)
            #Key(filter_key).
            data = table.query(KeyConditionExpression=filtering_exp)
        else:
            data = table.query()

        if self.logger != None:
            self.logger.info('Query Table with Key=%s and value=%s: %s' % (filter_key, filter_value, table))
            self.logger_info('result.count = %s' % len(data))
            if len(result) > 0:
                self.logger_info('result[0] = %s' % data[0])

        return data

    '''
    Put item: {'ResponseMetadata': {'HTTPHeaders': {'x-amz-crc32': '2745614147', 'connection': 'keep-alive', 'server': 'Server', 'date': 'Fri, 07 Dec 2018 07:24:58 GMT', 'x-amzn-requestid': 'F3RRH4E1QA0IV4B1D0SCCMAELFVV4KQNSO5AEMVJF66Q9ASUAAJG', 'content-type': 'application/x-amz-json-1.0', 'content-length': '2'}, 'RetryAttempts': 0, 'RequestId': 'F3RRH4E1QA0IV4B1D0SCCMAELFVV4KQNSO5AEMVJF66Q9ASUAAJG', 'HTTPStatusCode': 200}}
    '''
    def put_item(self, table_name, item_dict):
        table = self.get_table(table_name)
        response = table.put_item(
            TableName=table_name,
            Item=item_dict
        )
        
        if self.logger != None:
            self.logger.info('Put item to Table %s with response = %s' % (table_name, response))

        return response

    def iterate_put_items(self, table_name, item_dict_list):
        table = self.get_table(table_name)

        if self.logger != None:
            self.logger.info('Iteratively put %s items to Table %s' % (len(item_dict_list), table_name))

        response = ''
        for item_dict in item_dict_list:
            response = table.put_item(
                TableName=table_name,
                Item=item_dict
            )
            if self.logger != None and response['ResponseMetadata']['HTTPStatusCode']!=200:
                self.logger.warn('Put item unsuccessfully with item = %s and response = %s' % (item_dict, response))

        if self.logger != None:
             self.logger.info('Put last item with response = %s' % response)

        return response
